import * as plugins from '../../plugins.js';
import { HttpProxy } from '../http-proxy/index.js';
import { setupBidirectionalForwarding } from '../../core/utils/socket-utils.js';
import type { IConnectionRecord } from './models/interfaces.js';
import type { IRouteConfig } from './models/route-types.js';
import { WrappedSocket } from '../../core/models/wrapped-socket.js';
import type { SmartProxy } from './smart-proxy.js';

export class HttpProxyBridge {
  private httpProxy: HttpProxy | null = null;

  constructor(private smartProxy: SmartProxy) {}
  
  /**
   * Get the HttpProxy instance
   */
  public getHttpProxy(): HttpProxy | null {
    return this.httpProxy;
  }
  
  /**
   * Initialize HttpProxy instance
   */
  public async initialize(): Promise<void> {
    if (!this.httpProxy && this.smartProxy.settings.useHttpProxy && this.smartProxy.settings.useHttpProxy.length > 0) {
      const httpProxyOptions: any = {
        port: this.smartProxy.settings.httpProxyPort!,
        portProxyIntegration: true,
        logLevel: this.smartProxy.settings.enableDetailedLogging ? 'debug' : 'info'
      };

      this.httpProxy = new HttpProxy(httpProxyOptions);
      console.log(`Initialized HttpProxy on port ${this.smartProxy.settings.httpProxyPort}`);

      // Apply route configurations to HttpProxy
      await this.syncRoutesToHttpProxy(this.smartProxy.settings.routes || []);
    }
  }
  
  /**
   * Sync routes to HttpProxy
   */
  public async syncRoutesToHttpProxy(routes: IRouteConfig[]): Promise<void> {
    if (!this.httpProxy) return;
    
    // Convert routes to HttpProxy format
    const httpProxyConfigs = routes
      .filter(route => {
        // Check if this route matches any of the specified network proxy ports
        const routePorts = Array.isArray(route.match.ports) 
          ? route.match.ports 
          : [route.match.ports];
        
        return routePorts.some(port => 
          this.smartProxy.settings.useHttpProxy?.includes(port)
        );
      })
      .map(route => this.routeToHttpProxyConfig(route));
    
    // Apply configurations to HttpProxy
    await this.httpProxy.updateRouteConfigs(httpProxyConfigs);
  }
  
  /**
   * Convert route to HttpProxy configuration
   */
  private routeToHttpProxyConfig(route: IRouteConfig): any {
    // Convert route to HttpProxy domain config format
    let domain = '*';
    if (route.match.domains) {
      if (Array.isArray(route.match.domains)) {
        domain = route.match.domains[0] || '*';
      } else {
        domain = route.match.domains;
      }
    }
    
    return {
      ...route,  // Keep the original route structure
      match: {
        ...route.match,
        domains: domain  // Ensure domains is always set for HttpProxy
      }
    };
  }
  
  /**
   * Check if connection should use HttpProxy
   */
  public shouldUseHttpProxy(connection: IConnectionRecord, routeMatch: any): boolean {
    // Only use HttpProxy for TLS termination
    return (
      routeMatch.route.action.tls?.mode === 'terminate' ||
      routeMatch.route.action.tls?.mode === 'terminate-and-reencrypt'
    ) && this.httpProxy !== null;
  }
  
  /**
   * Forward connection to HttpProxy
   */
  public async forwardToHttpProxy(
    connectionId: string,
    socket: plugins.net.Socket | WrappedSocket,
    record: IConnectionRecord,
    initialChunk: Buffer,
    httpProxyPort: number,
    cleanupCallback: (reason: string) => void
  ): Promise<void> {
    if (!this.httpProxy) {
      throw new Error('HttpProxy not initialized');
    }
    
    const proxySocket = new plugins.net.Socket();
    
    await new Promise<void>((resolve, reject) => {
      proxySocket.connect(httpProxyPort, 'localhost', () => {
        console.log(`[${connectionId}] Connected to HttpProxy for termination`);
        resolve();
      });
      
      proxySocket.on('error', reject);
    });
    
    // Send client IP information header first (custom protocol)
    // Format: "CLIENT_IP:<ip>\r\n"
    const clientIPHeader = Buffer.from(`CLIENT_IP:${record.remoteIP}\r\n`);
    proxySocket.write(clientIPHeader);
    
    // Send initial chunk if present
    if (initialChunk) {
      // Count the initial chunk bytes
      record.bytesReceived += initialChunk.length;
      if (this.smartProxy.metricsCollector) {
        this.smartProxy.metricsCollector.recordBytes(record.id, initialChunk.length, 0);
      }
      proxySocket.write(initialChunk);
    }
    
    // Use centralized bidirectional forwarding
    // Extract underlying socket if it's a WrappedSocket
    const underlyingSocket = socket instanceof WrappedSocket ? socket.socket : socket;
    
    setupBidirectionalForwarding(underlyingSocket, proxySocket, {
      onClientData: (chunk) => {
        // Update stats - this is the ONLY place bytes are counted for HttpProxy connections
        if (record) {
          record.bytesReceived += chunk.length;
          if (this.smartProxy.metricsCollector) {
            this.smartProxy.metricsCollector.recordBytes(record.id, chunk.length, 0);
          }
        }
      },
      onServerData: (chunk) => {
        // Update stats - this is the ONLY place bytes are counted for HttpProxy connections
        if (record) {
          record.bytesSent += chunk.length;
          if (this.smartProxy.metricsCollector) {
            this.smartProxy.metricsCollector.recordBytes(record.id, 0, chunk.length);
          }
        }
      },
      onCleanup: (reason) => {
        cleanupCallback(reason);
      },
      enableHalfOpen: false // Close both when one closes (required for proxy chains)
    });
  }
  
  /**
   * Start HttpProxy
   */
  public async start(): Promise<void> {
    if (this.httpProxy) {
      await this.httpProxy.start();
    }
  }
  
  /**
   * Stop HttpProxy
   */
  public async stop(): Promise<void> {
    if (this.httpProxy) {
      await this.httpProxy.stop();
      this.httpProxy = null;
    }
  }
}